@@ -1,10 +1,14 @@
 # -*- coding: utf-8 -*-
+import datetime
 import tempfile
 import os
+
+from django.contrib.auth.models import User
+from django.contrib.contenttypes import generic
+from django.contrib.contenttypes.models import ContentType
 from django.core.files.storage import FileSystemStorage
 from django.db import models
-from django.contrib import admin
-from django.core.mail import EmailMessage
+
 
 class Section(models.Model):
     """
@@ -13,6 +17,7 @@ class Section(models.Model):
     """
     name = models.CharField(max_length=100)
 
+
 class Article(models.Model):
     """
     A simple article to test admin views. Test backwards compatibility.
@@ -20,7 +25,7 @@ class Article(models.Model):
     title = models.CharField(max_length=100)
     content = models.TextField()
     date = models.DateTimeField()
-    section = models.ForeignKey(Section)
+    section = models.ForeignKey(Section, null=True, blank=True)
 
     def __unicode__(self):
         return self.title
@@ -28,6 +33,8 @@ class Article(models.Model):
     def model_year(self):
         return self.date.year
     model_year.admin_order_field = 'date'
+    model_year.short_description = ''
+
 
 class Book(models.Model):
     """
@@ -38,6 +45,7 @@ class Book(models.Model):
     def __unicode__(self):
         return self.name
 
+
 class Promo(models.Model):
     name = models.CharField(max_length=100, verbose_name=u'¿Name?')
     book = models.ForeignKey(Book)
@@ -45,6 +53,7 @@ class Promo(models.Model):
     def __unicode__(self):
         return self.name
 
+
 class Chapter(models.Model):
     title = models.CharField(max_length=100, verbose_name=u'¿Title?')
     content = models.TextField()
@@ -54,7 +63,9 @@ class Chapter(models.Model):
         return self.title
 
     class Meta:
-        verbose_name = u'¿Chapter?'
+        # Use a utf-8 bytestring to ensure it works (see #11710)
+        verbose_name = '¿Chapter?'
+
 
 class ChapterXtra1(models.Model):
     chap = models.OneToOneField(Chapter, verbose_name=u'¿Chap?')
@@ -63,6 +74,7 @@ class ChapterXtra1(models.Model):
     def __unicode__(self):
         return u'¿Xtra1: %s' % self.xtra
 
+
 class ChapterXtra2(models.Model):
     chap = models.OneToOneField(Chapter, verbose_name=u'¿Chap?')
     xtra = models.CharField(max_length=100, verbose_name=u'¿Xtra?')
@@ -70,52 +82,15 @@ class ChapterXtra2(models.Model):
     def __unicode__(self):
         return u'¿Xtra2: %s' % self.xtra
 
-def callable_year(dt_value):
-    return dt_value.year
-callable_year.admin_order_field = 'date'
 
-class ArticleInline(admin.TabularInline):
-    model = Article
+class RowLevelChangePermissionModel(models.Model):
+    name = models.CharField(max_length=100, blank=True)
 
-class ChapterInline(admin.TabularInline):
-    model = Chapter
-
-class ArticleAdmin(admin.ModelAdmin):
-    list_display = ('content', 'date', callable_year, 'model_year', 'modeladmin_year')
-    list_filter = ('date',)
-
-    def changelist_view(self, request):
-        "Test that extra_context works"
-        return super(ArticleAdmin, self).changelist_view(
-            request, extra_context={
-                'extra_var': 'Hello!'
-            }
-        )
-
-    def modeladmin_year(self, obj):
-        return obj.date.year
-    modeladmin_year.admin_order_field = 'date'
 
 class CustomArticle(models.Model):
     content = models.TextField()
     date = models.DateTimeField()
 
-class CustomArticleAdmin(admin.ModelAdmin):
-    """
-    Tests various hooks for using custom templates and contexts.
-    """
-    change_list_template = 'custom_admin/change_list.html'
-    change_form_template = 'custom_admin/change_form.html'
-    object_history_template = 'custom_admin/object_history.html'
-    delete_confirmation_template = 'custom_admin/delete_confirmation.html'
-
-    def changelist_view(self, request):
-        "Test that extra_context works"
-        return super(CustomArticleAdmin, self).changelist_view(
-            request, extra_context={
-                'extra_var': 'Hello!'
-            }
-        )
 
 class ModelWithStringPrimaryKey(models.Model):
     id = models.CharField(max_length=255, primary_key=True)
@@ -123,20 +98,52 @@ class ModelWithStringPrimaryKey(models.Model):
     def __unicode__(self):
         return self.id
 
+
 class Color(models.Model):
     value = models.CharField(max_length=10)
     warm = models.BooleanField()
     def __unicode__(self):
         return self.value
 
+# we replicate Color to register with another ModelAdmin
+class Color2(Color):
+    class Meta:
+        proxy = True
+
 class Thing(models.Model):
     title = models.CharField(max_length=20)
     color = models.ForeignKey(Color, limit_choices_to={'warm': True})
+    pub_date = models.DateField(blank=True, null=True)
+    def __unicode__(self):
+        return self.title
+
+
+class Actor(models.Model):
+    name = models.CharField(max_length=50)
+    age = models.IntegerField()
+    def __unicode__(self):
+        return self.name
+
+
+class Inquisition(models.Model):
+    expected = models.BooleanField()
+    leader = models.ForeignKey(Actor)
+    country = models.CharField(max_length=20)
+
+    def __unicode__(self):
+        return u"by %s from %s" % (self.leader, self.country)
+
+
+class Sketch(models.Model):
+    title = models.CharField(max_length=100)
+    inquisition = models.ForeignKey(Inquisition, limit_choices_to={'leader__name': 'Palin',
+                                                                   'leader__age': 27,
+                                                                   'expected': False,
+                                                                   })
+
     def __unicode__(self):
         return self.title
 
-class ThingAdmin(admin.ModelAdmin):
-    list_filter = ('color',)
 
 class Fabric(models.Model):
     NG_CHOICES = (
@@ -149,9 +156,6 @@ class Fabric(models.Model):
     )
     surface = models.CharField(max_length=20, choices=NG_CHOICES)
 
-class FabricAdmin(admin.ModelAdmin):
-    list_display = ('surface',)
-    list_filter = ('surface',)
 
 class Person(models.Model):
     GENDER_CHOICES = (
@@ -160,21 +164,12 @@ class Person(models.Model):
     )
     name = models.CharField(max_length=100)
     gender = models.IntegerField(choices=GENDER_CHOICES)
+    age = models.IntegerField(default=21)
     alive = models.BooleanField()
 
     def __unicode__(self):
         return self.name
 
-    class Meta:
-        ordering = ["id"]
-
-class PersonAdmin(admin.ModelAdmin):
-    list_display = ('name', 'gender', 'alive')
-    list_editable = ('gender', 'alive')
-    list_filter = ('gender',)
-    search_fields = (u'name',)
-    ordering = ["id"]
-    save_as = True
 
 class Persona(models.Model):
     """
@@ -185,6 +180,7 @@ class Persona(models.Model):
     def __unicode__(self):
         return self.name
 
+
 class Account(models.Model):
     """
     A simple, generic account encapsulating the information shared by all
@@ -197,27 +193,16 @@ class Account(models.Model):
     def __unicode__(self):
         return "%s: %s" % (self.servicename, self.username)
 
+
 class FooAccount(Account):
     """A service-specific account of type Foo."""
     servicename = u'foo'
 
+
 class BarAccount(Account):
     """A service-specific account of type Bar."""
     servicename = u'bar'
 
-class FooAccountAdmin(admin.StackedInline):
-    model = FooAccount
-    extra = 1
-
-class BarAccountAdmin(admin.StackedInline):
-    model = BarAccount
-    extra = 1
-
-class PersonaAdmin(admin.ModelAdmin):
-    inlines = (
-        FooAccountAdmin,
-        BarAccountAdmin
-    )
 
 class Subscriber(models.Model):
     name = models.CharField(blank=False, max_length=80)
@@ -226,105 +211,58 @@ class Subscriber(models.Model):
     def __unicode__(self):
         return "%s (%s)" % (self.name, self.email)
 
-class SubscriberAdmin(admin.ModelAdmin):
-    actions = ['mail_admin']
-
-    def mail_admin(self, request, selected):
-        EmailMessage(
-            'Greetings from a ModelAdmin action',
-            'This is the test email from a admin action',
-            'from@example.com',
-            ['to@example.com']
-        ).send()
 
 class ExternalSubscriber(Subscriber):
     pass
 
+
 class OldSubscriber(Subscriber):
     pass
 
-def external_mail(modeladmin, request, selected):
-    EmailMessage(
-        'Greetings from a function action',
-        'This is the test email from a function action',
-        'from@example.com',
-        ['to@example.com']
-    ).send()
-
-def redirect_to(modeladmin, request, selected):
-    from django.http import HttpResponseRedirect
-    return HttpResponseRedirect('/some-where-else/')
-
-class ExternalSubscriberAdmin(admin.ModelAdmin):
-    actions = [external_mail, redirect_to]
 
 class Media(models.Model):
     name = models.CharField(max_length=60)
 
+
 class Podcast(Media):
     release_date = models.DateField()
 
-class PodcastAdmin(admin.ModelAdmin):
-    list_display = ('name', 'release_date')
-    list_editable = ('release_date',)
+    class Meta:
+        ordering = ('release_date',) # overridden in PodcastAdmin
 
-    ordering = ('name',)
 
 class Vodcast(Media):
     media = models.OneToOneField(Media, primary_key=True, parent_link=True)
     released = models.BooleanField(default=False)
 
-class VodcastAdmin(admin.ModelAdmin):
-    list_display = ('name', 'released')
-    list_editable = ('released',)
-
-    ordering = ('name',)
 
 class Parent(models.Model):
     name = models.CharField(max_length=128)
 
+
 class Child(models.Model):
     parent = models.ForeignKey(Parent, editable=False)
     name = models.CharField(max_length=30, blank=True)
 
-class ChildInline(admin.StackedInline):
-    model = Child
-
-class ParentAdmin(admin.ModelAdmin):
-    model = Parent
-    inlines = [ChildInline]
 
 class EmptyModel(models.Model):
     def __unicode__(self):
         return "Primary key = %s" % self.id
 
-class EmptyModelAdmin(admin.ModelAdmin):
-    def queryset(self, request):
-        return super(EmptyModelAdmin, self).queryset(request).filter(pk__gt=1)
 
-class OldSubscriberAdmin(admin.ModelAdmin):
-    actions = None
-
-temp_storage = FileSystemStorage(tempfile.mkdtemp())
+temp_storage = FileSystemStorage(tempfile.mkdtemp(dir=os.environ['DJANGO_TEST_TEMP_DIR']))
 UPLOAD_TO = os.path.join(temp_storage.location, 'test_upload')
 
+
 class Gallery(models.Model):
     name = models.CharField(max_length=100)
 
+
 class Picture(models.Model):
     name = models.CharField(max_length=100)
     image = models.FileField(storage=temp_storage, upload_to='test_upload')
     gallery = models.ForeignKey(Gallery, related_name="pictures")
 
-class PictureInline(admin.TabularInline):
-    model = Picture
-    extra = 1
-
-class GalleryAdmin(admin.ModelAdmin):
-    inlines = [PictureInline]
-
-class PictureAdmin(admin.ModelAdmin):
-    pass
 
 class Language(models.Model):
     iso = models.CharField(max_length=5, primary_key=True)
@@ -335,70 +273,60 @@ class Language(models.Model):
     class Meta:
         ordering = ('iso',)
 
-class LanguageAdmin(admin.ModelAdmin):
-    list_display = ['iso', 'shortlist', 'english_name', 'name']
-    list_editable = ['shortlist']
 
 # a base class for Recommender and Recommendation
 class Title(models.Model):
     pass
 
+
 class TitleTranslation(models.Model):
     title = models.ForeignKey(Title)
     text = models.CharField(max_length=100)
 
+
 class Recommender(Title):
     pass
 
+
 class Recommendation(Title):
     recommender = models.ForeignKey(Recommender)
 
-class RecommendationAdmin(admin.ModelAdmin):
-    search_fields = ('titletranslation__text', 'recommender__titletranslation__text',)
 
 class Collector(models.Model):
     name = models.CharField(max_length=100)
 
+
 class Widget(models.Model):
     owner = models.ForeignKey(Collector)
     name = models.CharField(max_length=100)
 
+
 class DooHickey(models.Model):
     code = models.CharField(max_length=10, primary_key=True)
     owner = models.ForeignKey(Collector)
     name = models.CharField(max_length=100)
 
+
 class Grommet(models.Model):
     code = models.AutoField(primary_key=True)
     owner = models.ForeignKey(Collector)
     name = models.CharField(max_length=100)
 
+
 class Whatsit(models.Model):
     index = models.IntegerField(primary_key=True)
     owner = models.ForeignKey(Collector)
     name = models.CharField(max_length=100)
 
+
 class Doodad(models.Model):
     name = models.CharField(max_length=100)
 
+
 class FancyDoodad(Doodad):
     owner = models.ForeignKey(Collector)
     expensive = models.BooleanField(default=True)
 
-class WidgetInline(admin.StackedInline):
-    model = Widget
-
-class DooHickeyInline(admin.StackedInline):
-    model = DooHickey
-
-class GrommetInline(admin.StackedInline):
-    model = Grommet
-
-class WhatsitInline(admin.StackedInline):
-    model = Whatsit
-
-class FancyDoodadInline(admin.StackedInline):
-    model = FancyDoodad
 
 class Category(models.Model):
     collector = models.ForeignKey(Collector)
@@ -410,49 +338,321 @@ class Category(models.Model):
     def __unicode__(self):
         return u'%s:o%s' % (self.id, self.order)
 
-class CategoryAdmin(admin.ModelAdmin):
-    list_display = ('id', 'collector', 'order')
-    list_editable = ('order',)
 
-class CategoryInline(admin.StackedInline):
-    model = Category
+class Link(models.Model):
+    posted = models.DateField(
+        default=lambda: datetime.date.today() - datetime.timedelta(days=7)
+    )
+    url = models.URLField()
+    post = models.ForeignKey("Post")
 
-class CollectorAdmin(admin.ModelAdmin):
-    inlines = [WidgetInline, DooHickeyInline, GrommetInline, WhatsitInline, FancyDoodadInline, CategoryInline]
 
-admin.site.register(Article, ArticleAdmin)
-admin.site.register(CustomArticle, CustomArticleAdmin)
-admin.site.register(Section, save_as=True, inlines=[ArticleInline])
-admin.site.register(ModelWithStringPrimaryKey)
-admin.site.register(Color)
-admin.site.register(Thing, ThingAdmin)
-admin.site.register(Person, PersonAdmin)
-admin.site.register(Persona, PersonaAdmin)
-admin.site.register(Subscriber, SubscriberAdmin)
-admin.site.register(ExternalSubscriber, ExternalSubscriberAdmin)
-admin.site.register(OldSubscriber, OldSubscriberAdmin)
-admin.site.register(Podcast, PodcastAdmin)
-admin.site.register(Vodcast, VodcastAdmin)
-admin.site.register(Parent, ParentAdmin)
-admin.site.register(EmptyModel, EmptyModelAdmin)
-admin.site.register(Fabric, FabricAdmin)
-admin.site.register(Gallery, GalleryAdmin)
-admin.site.register(Picture, PictureAdmin)
-admin.site.register(Language, LanguageAdmin)
-admin.site.register(Recommendation, RecommendationAdmin)
-admin.site.register(Recommender)
-admin.site.register(Collector, CollectorAdmin)
-admin.site.register(Category, CategoryAdmin)
+class PrePopulatedPost(models.Model):
+    title = models.CharField(max_length=100)
+    published = models.BooleanField()
+    slug = models.SlugField()
 
-# We intentionally register Promo and ChapterXtra1 but not Chapter nor ChapterXtra2.
-# That way we cover all four cases:
-#     related ForeignKey object registered in admin
-#     related ForeignKey object not registered in admin
-#     related OneToOne object registered in admin
-#     related OneToOne object not registered in admin
-# when deleting Book so as exercise all four troublesome (w.r.t escaping
-# and calling force_unicode to avoid problems on Python 2.3) paths through
-# contrib.admin.util's get_deleted_objects function.
-admin.site.register(Book, inlines=[ChapterInline])
-admin.site.register(Promo)
-admin.site.register(ChapterXtra1)
+
+class PrePopulatedSubPost(models.Model):
+    post = models.ForeignKey(PrePopulatedPost)
+    subtitle = models.CharField(max_length=100)
+    subslug = models.SlugField()
+
+
+class Post(models.Model):
+    title = models.CharField(max_length=100, help_text="Some help text for the title (with unicode ŠĐĆŽćžšđ)")
+    content = models.TextField(help_text="Some help text for the content (with unicode ŠĐĆŽćžšđ)")
+    posted = models.DateField(
+            default=datetime.date.today,
+            help_text="Some help text for the date (with unicode ŠĐĆŽćžšđ)"
+    )
+    public = models.NullBooleanField()
+
+    def awesomeness_level(self):
+        return "Very awesome."
+
+
+class Gadget(models.Model):
+    name = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return self.name
+
+
+class Villain(models.Model):
+    name = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return self.name
+
+
+class SuperVillain(Villain):
+    pass
+
+
+class FunkyTag(models.Model):
+    "Because we all know there's only one real use case for GFKs."
+    name = models.CharField(max_length=25)
+    content_type = models.ForeignKey(ContentType)
+    object_id = models.PositiveIntegerField()
+    content_object = generic.GenericForeignKey('content_type', 'object_id')
+
+    def __unicode__(self):
+        return self.name
+
+
+class Plot(models.Model):
+    name = models.CharField(max_length=100)
+    team_leader = models.ForeignKey(Villain, related_name='lead_plots')
+    contact = models.ForeignKey(Villain, related_name='contact_plots')
+    tags = generic.GenericRelation(FunkyTag)
+
+    def __unicode__(self):
+        return self.name
+
+
+class PlotDetails(models.Model):
+    details = models.CharField(max_length=100)
+    plot = models.OneToOneField(Plot)
+
+    def __unicode__(self):
+        return self.details
+
+
+class SecretHideout(models.Model):
+    """ Secret! Not registered with the admin! """
+    location = models.CharField(max_length=100)
+    villain = models.ForeignKey(Villain)
+
+    def __unicode__(self):
+        return self.location
+
+
+class SuperSecretHideout(models.Model):
+    """ Secret! Not registered with the admin! """
+    location = models.CharField(max_length=100)
+    supervillain = models.ForeignKey(SuperVillain)
+
+    def __unicode__(self):
+        return self.location
+
+
+class CyclicOne(models.Model):
+    name = models.CharField(max_length=25)
+    two = models.ForeignKey('CyclicTwo')
+
+    def __unicode__(self):
+        return self.name
+
+
+class CyclicTwo(models.Model):
+    name = models.CharField(max_length=25)
+    one = models.ForeignKey(CyclicOne)
+
+    def __unicode__(self):
+        return self.name
+
+
+class Topping(models.Model):
+    name = models.CharField(max_length=20)
+
+
+class Pizza(models.Model):
+    name = models.CharField(max_length=20)
+    toppings = models.ManyToManyField('Topping')
+
+
+class Album(models.Model):
+    owner = models.ForeignKey(User)
+    title = models.CharField(max_length=30)
+
+
+class Employee(Person):
+    code = models.CharField(max_length=20)
+
+
+class WorkHour(models.Model):
+    datum = models.DateField()
+    employee = models.ForeignKey(Employee)
+
+
+class Question(models.Model):
+    question = models.CharField(max_length=20)
+
+
+class Answer(models.Model):
+    question = models.ForeignKey(Question, on_delete=models.PROTECT)
+    answer = models.CharField(max_length=20)
+
+    def __unicode__(self):
+        return self.answer
+
+
+class Reservation(models.Model):
+    start_date = models.DateTimeField()
+    price = models.IntegerField()
+
+
+DRIVER_CHOICES = (
+    (u'bill', 'Bill G'),
+    (u'steve', 'Steve J'),
+)
+
+RESTAURANT_CHOICES = (
+    (u'indian', u'A Taste of India'),
+    (u'thai', u'Thai Pography'),
+    (u'pizza', u'Pizza Mama'),
+)
+
+
+class FoodDelivery(models.Model):
+    reference = models.CharField(max_length=100)
+    driver = models.CharField(max_length=100, choices=DRIVER_CHOICES, blank=True)
+    restaurant = models.CharField(max_length=100, choices=RESTAURANT_CHOICES, blank=True)
+
+    class Meta:
+        unique_together = (("driver", "restaurant"),)
+
+
+class Paper(models.Model):
+    title = models.CharField(max_length=30)
+    author = models.CharField(max_length=30, blank=True, null=True)
+
+
+class CoverLetter(models.Model):
+    author = models.CharField(max_length=30)
+    date_written = models.DateField(null=True, blank=True)
+
+    def __unicode__(self):
+        return self.author
+
+
+class Story(models.Model):
+    title = models.CharField(max_length=100)
+    content = models.TextField()
+
+
+class OtherStory(models.Model):
+    title = models.CharField(max_length=100)
+    content = models.TextField()
+
+
+class ComplexSortedPerson(models.Model):
+    name = models.CharField(max_length=100)
+    age = models.PositiveIntegerField()
+    is_employee = models.NullBooleanField()
+
+class PrePopulatedPostLargeSlug(models.Model):
+    """
+    Regression test for #15938: a large max_length for the slugfield must not
+    be localized in prepopulated_fields_js.html or it might end up breaking
+    the javascript (ie, using THOUSAND_SEPARATOR ends up with maxLength=1,000)
+    """
+    title = models.CharField(max_length=100)
+    published = models.BooleanField()
+    slug = models.SlugField(max_length=1000)
+
+class AdminOrderedField(models.Model):
+    order = models.IntegerField()
+    stuff = models.CharField(max_length=200)
+
+class AdminOrderedModelMethod(models.Model):
+    order = models.IntegerField()
+    stuff = models.CharField(max_length=200)
+    def some_order(self):
+        return self.order
+    some_order.admin_order_field = 'order'
+
+class AdminOrderedAdminMethod(models.Model):
+    order = models.IntegerField()
+    stuff = models.CharField(max_length=200)
+
+class AdminOrderedCallable(models.Model):
+    order = models.IntegerField()
+    stuff = models.CharField(max_length=200)
+
+class Report(models.Model):
+    title = models.CharField(max_length=100)
+
+    def __unicode__(self):
+        return self.title
+
+
+class MainPrepopulated(models.Model):
+    name = models.CharField(max_length=100)
+    pubdate = models.DateField()
+    status = models.CharField(
+        max_length=20,
+        choices=(('option one', 'Option One'),
+                 ('option two', 'Option Two')))
+    slug1 = models.SlugField()
+    slug2 = models.SlugField()
+
+class RelatedPrepopulated(models.Model):
+    parent = models.ForeignKey(MainPrepopulated)
+    name = models.CharField(max_length=75)
+    pubdate = models.DateField()
+    status = models.CharField(
+        max_length=20,
+        choices=(('option one', 'Option One'),
+                 ('option two', 'Option Two')))
+    slug1 = models.SlugField(max_length=50)
+    slug2 = models.SlugField(max_length=60)
+
+
+class UnorderedObject(models.Model):
+    """
+    Model without any defined `Meta.ordering`.
+    Refs #16819.
+    """
+    name = models.CharField(max_length=255)
+    bool = models.BooleanField(default=True)
+
+
+# Models for #23329
+class ReferencedByParent(models.Model):
+    name = models.CharField(max_length=20, unique=True)
+
+
+class ParentWithFK(models.Model):
+    fk = models.ForeignKey(
+        ReferencedByParent, to_field='name', related_name='hidden+',
+    )
+
+
+class ChildOfReferer(ParentWithFK):
+    pass
+
+
+class M2MReference(models.Model):
+    ref = models.ManyToManyField('self')
+
+
+# Models for #23431
+class ReferencedByInline(models.Model):
+    name = models.CharField(max_length=20, unique=True)
+
+
+class InlineReference(models.Model):
+    fk = models.ForeignKey(
+        ReferencedByInline, to_field='name', related_name='hidden+',
+    )
+
+
+class InlineReferer(models.Model):
+    refs = models.ManyToManyField(InlineReference)
+
+
+# Models for #23604
+class Recipe(models.Model):
+    pass
+
+
+class Ingredient(models.Model):
+    recipes = models.ManyToManyField(Recipe)
+
+
+# Model for #23839
+class NotReferenced(models.Model):
+    # Don't point any FK at this model.
+    pass
